#!/usr/bin/python
#Dummy functions. To complete once details about encryption are finalized
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Signature import PKCS1_PSS
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from Crypto import Random
import os
import base64
#from PBKDF2 import PBKDF2

# Get something from text with a length of a multiple of 16
# Because AES needs multiples of 16
def pad(text):
	return text+(16-len(text)%16)*chr(16-len(text)%16)

# return padded text to original size
def unpad(text):
	return text[0:-ord(text[-1])]

# PKCS1_OAEP wants short input, so we need to break things into pieces
def split(text, length):
	return [text[n:n+length] for n in range(0, len(text), length)]

def escape(string):
	return base64.b64encode(string)

def unescape(string):
	return base64.b64decode(string)

# Symmetric encryption
def decrypt(encrypted_text, key):
	key = unescape(key)
	encrypted_text = unescape(encrypted_text)
	iv = encrypted_text[:16]
	encrypted=encrypted_text[16:]
	decrypter = AES.new(key, AES.MODE_CBC, iv)
	decrypted_text = decrypter.decrypt(encrypted)
	return unpad(decrypted_text)

def encrypt(unencrypted_text, key):
	key = unescape(key)
	iv = os.urandom(16)
	encrypter = AES.new(key, AES.MODE_CBC, iv)
	encrypted_text = encrypter.encrypt(pad(unencrypted_text))
	return escape(iv+encrypted_text)

# Public-key encryption

RSA_KEY_SIZE = 2048

def pubkey_encrypt(unencrypted_text, public_key):
	# PKCS1_OAEP is an RSA protocol. The pycrypto docs said not
	# to use the public key directly.
	encrypter = PKCS1_OAEP.new(public_key)
	encrypted_text = "".join([encrypter.encrypt(piece)
				  for piece in split(unencrypted_text, RSA_KEY_SIZE/8 - 50)])
	return escape(encrypted_text)

def pubkey_decrypt(encrypted_text, private_key):
	encrypted_text = unescape(encrypted_text)
	decrypter = PKCS1_OAEP.new(private_key)
	decrypted_text = "".join([decrypter.decrypt(piece)
				  for piece in split(encrypted_text, RSA_KEY_SIZE/8)])
	return decrypted_text

# Hashes

def hash(text):
	return SHA256.new(text).hexdigest() 

# Signatures

def sign(text, private_key):
	hash_text = hash(text)
	signer = PKCS1_PSS.new(private_key)
	return signer.sign(hash)

def verify(text, signature, public_key):
	hash_for_text = hash(text)
	verifier = PKCS1_PSS.new(public_key)
	return verifier.verify(hash_for_text, signature)

# Key management

def generate_key():
	return escape(Random.get_random_bytes(32))

def generate_pubkey():
	return RSA.generate(RSA_KEY_SIZE)

def import_key(key, passphrase=None):
	return RSA.importKey(key, passphrase)

def export_key(key, passphrase=None):
	return key.exportKey(passphrase=passphrase)

def pubkey_to_symmetric_key(private_key):
	return escape(SHA256.new(export_key(private_key)).digest())

'''test = "this is 12 c"
random_generator = Random.new().read
key = RSA.generate(1024, random_generator)
print len(test)
encrypted = pubkey_encrypt(test, key.publickey())
print encrypted
print pubkey_decrypt(encrypted, key) '''
