import json
from efs.common.exceptions import InvalidCommandError
from efs.common.exceptions import InvalidRequestError
from efs.common.exceptions import InvalidResponseError

class Command(object):

	def __init__(self, command=None, args=None, text_command=None, object_command=None):
		#JSON constructor (Use Command.create_from_text())
		if text_command != None and object_command == None:
			try:
				json_command = json.loads(text_command)
				self.command = json_command["command"]
				self.args = json_command["args"]
			except:
				raise InvalidCommandError
		#Object constructor (Use Command.create_from_object())
		elif object_command != None and text_command == None:
			try:
				self.command = object_command["command"]
				self.args = object_command["args"]
			except:
				raise InvalidCommandError
		#Object constructor (Use Command.create() or simply Command())
		else:
			self.command = command
			self.args = args

	@classmethod
	def create(cls, command, args):
		return cls(command, args)

	@classmethod
	def create_from_text(cls, text_command):
		return cls(text_command=text_command)

	@classmethod
	def create_from_object(cls, object_command):
		return cls(object_command=object_command)

	def to_json(self):
		return json.dumps(self.__dict__)


class Request(object):

	def __init__(self, command=None, token=None, timestamp=None, user=None, text_request=None, data=None):
		#JSON constructor (Use Request.create_from_text())
		if text_request != None:
			try:
				json_request = json.loads(text_request)
				self.command = Command.create_from_object(json_request["command"])
				self.token = json_request["token"]
				self.timestamp = float(json_request["timestamp"])
				self.user = json_request["user"]
				self.data = json_request["data"]
			except:
				raise InvalidRequestError
		#Parameter constructure (Use Request.create() or Request())
		else:
			self.command = command
			self.token = token
			self.timestamp = timestamp
			self.user = user
			self.data = data
			
	@classmethod
	def create(cls, command, token, timestamp, data=""):
		return cls(command=command, token=token, timestamp=timestamp, data=data)
 
	@classmethod
	def create_from_text(cls, text_request):
		return cls(text_request=text_request)

	def to_json(self):
		o = dict(self.__dict__)
		o["command"] = o["command"].__dict__
		return json.dumps(o)


class Response(object):

	def __init__(self, code=None, data=None, text_response=None):
		#JSON constructor (Use Response.create_from_text())
		if text_response != None:
			json_response = json.loads(text_response)
			try:
				self.code = json_response["code"]
				self.data = json_response["data"]
			except:
				raise InvalidResponseError
		#Parameter constructure (Use Response.create() or Response())
		else:
			self.code = code
			self.data = data

	@classmethod
	def create(cls, code, data):
		return cls(code=code, data=data)

	@classmethod
	def create_from_text(cls, text_response):
		return cls(text_response=text_response)

	def to_json(self):
		return json.dumps(self.__dict__)
