class AuthorizationService(object):

	def __init__(self, mongo_client):
		self.login_db = mongo_client.login
		self.users = self.login_db.users
		self.sessions = self.login_db.sessions
		self.groups = self.login_db.groups

	def authorize(self, efs_request):
		if efs_request.command.command in ["get_server_public_key", "register", "login"]:
			# This command doesn't require authentication
			return True
		if self.check_timestamp(efs_request.user, efs_request.timestamp) \
			and self.check_token(efs_request.user, efs_request.token):
			if efs_request.command.command == "chpasswd":
				return self.authorize_chpasswd(efs_request)
			elif efs_request.command.command == "mkgroup":
				return self.authorize_mkgroup(efs_request)
			elif efs_request.command.command == "rmgroup" or \
				efs_request.command.command == "adduser" or \
				efs_request.command.command == "rmuser":
				return self.authorize_group_owner(efs_request)
			elif efs_request.command.command == "get_user_struct" or \
				efs_request.command.command == "update_user_struct":
				return self.authorize_user_struct(efs_request)
			elif efs_request.command.command == "get_group_struct" or \
				efs_request.command.command == "update_group_struct":
				return self.authorize_group_struct(efs_request)
			else:
				return True
		else:
			return False

	def authorize_chpasswd(self, efs_request):
		try:
			if efs_request.user == efs_request.command.args["user"]:
				return True
			else:
				return False
		except:
			return False

	def authorize_mkgroup(self, efs_request):
		try:
			if efs_request.user == efs_request.command.args["owner"]:
				return True
			else:
				return False
		except:
			return False		

	def authorize_group_owner(self, efs_request):
		try:
			group_owner = self.get_group_owner(efs_request.command.args["group"])
			if efs_request.user == group_owner:
				return True
			else:
				return False
		except:
			return False

	def authorize_user_struct(self, efs_request):
		try:
			if efs_request.user == efs_request.command.args["user"]:
				return True
			else:
				return False
		except:
			return False


	def authorize_group_struct(self, efs_request):
		try:
			group = efs_request.command.args["group"]
			if self.user_in_group(efs_request.user, group) == True:
				return True
			return False
		except:
			return False

	def get_group_owner(self, group):
		group_record = self.groups.find({"group":group})
		if group_record.count() == 1:
			return group_record[0]["owner"]
		else:
			return None

	def user_in_group(self, user, group):
		owner = self.get_group_owner(group)
		if owner == user:
			return True
		group_record = self.groups.find({"group":group})
		if group_record.count == 1:
			members = group_record[0].members
			if user in members:
				return True
			else:
				return False
		else:
			return False

	def check_token(self, user, token):
		session_record = self.sessions.find({"user":user}).limit(1)
		if session_record.count() == 1:
			if session_record[0]["token"] == token:
				return True
			else:
				return False
		else:
			return False

	def check_timestamp(self, user, timestamp):
		session_record = self.sessions.find({"user":user})
		if session_record.count() == 1 and session_record[0]["timestamp"] < timestamp:
			return True
		else:
			return False
