import unittest
from pymongo import MongoClient

from efs.common.encryption import generate_pubkey
from efs.server.fs import init
from efs.server.keys import KeyService
from efs.server.login import LoginService

class KeyTests(unittest.TestCase):

	def setUp(self):
		self.mongo_client = MongoClient("127.0.0.1", 27017)
		self.users = self.mongo_client.login.users
		self.groups = self.mongo_client.login.groups
		self.sessions = self.mongo_client.login.sessions
		self.users.remove({})
		self.groups.remove({})
		self.sessions.remove({})
		init(self.mongo_client)
		self.login_service = LoginService(self.mongo_client)
		self.server_private_key = generate_pubkey()
		self.server_public_key = self.server_private_key.publickey().exportKey()
		self.server_private_key = self.server_private_key.exportKey()
		self.key_service = KeyService(self.mongo_client, self.server_public_key, self.server_private_key)
		self.setUpUsers()
		self.setUpGroups()

	def setUpUsers(self):
		self.user_private_key = generate_pubkey()
		self.user_public_key = self.user_private_key.publickey().exportKey()
		self.user_private_key = self.user_private_key.exportKey()
		self.login_service.register("user1", "1234", self.user_public_key)
		self.login_service.register("user2", "1234", self.user_public_key)
		self.login_service.register("user3", "1234", self.user_public_key)

	def setUpGroups(self):
		self.group_private_key = generate_pubkey()
		self.group_public_key = self.group_private_key.publickey().exportKey()
		self.group_private_key = self.group_private_key.exportKey()
		self.login_service.make_group("a", "user1", self.group_public_key, self.group_private_key)
		self.login_service.make_group("b", "user2", self.group_public_key, self.group_private_key)
		self.login_service.make_group("c", "user3", self.group_public_key, self.group_private_key)
		self.login_service.add_user("user2", "a", self.group_private_key)
		self.login_service.add_user("user3", "a", self.group_private_key)
		self.login_service.add_user("user1", "b", self.group_private_key)
		self.login_service.add_user("user3", "b", self.group_private_key)
		self.login_service.add_user("user1", "c", self.group_private_key)
		self.login_service.add_user("user2", "c", self.group_private_key)

	def test_get_server_public_key(self):
		key = self.key_service.get_server_public_key()
		self.assertEquals(key, self.server_public_key)

	def test_get_user_public_key(self):
		key = self.key_service.get_user_public_key("user1")
		self.assertEquals(self.user_public_key, key.exportKey())

	def test_get_group_public_key(self):
		key = self.key_service.get_group_public_key("a")
		self.assertEquals(self.group_public_key, key.exportKey())

	def test_get_group_private_key(self):
		key = self.key_service.get_group_private_key("b", "user2")
		self.assertEquals(self.group_private_key, key)

if __name__ == '__main__':
    unittest.main()